@@ -1,4 +1,4 @@
-# Copyright 2018, 2019, 2020 Blair Bonnett
+# Copyright 2018, 2019, 2020, 2021, 2022 Blair Bonnett
 #
 # Redistribution and use in source and binary forms, with or without
 # modification, are permitted provided that the following conditions are met:
@@ -36,7 +36,7 @@ consistent-looking plots.
 
 """
 
-__version__ = "1.4.0"
+__version__ = "1.7.0"
 
 # We don't import Matplotlib here as this brings in NumPy. In turn, NumPy
 # caches a reference to the io.open() method as part of its data loading
@@ -48,11 +48,12 @@ __version__ = "1.4.0"
 # different functions.
 
 import configparser
-import importlib
+import importlib.abc
 import inspect
 import io
 import os
 import os.path
+import pathlib
 import re
 import string
 import sys
@@ -410,6 +411,7 @@ def _config_reset():
             'figure_background': '',
             'axes_background': 'white',
             'extra_tracking': '',
+            'environment': '',
         },
 
         'paths': {
@@ -574,6 +576,8 @@ def _install_standard_file_trackers():
     import io
     builtins.open = _file_tracker(builtins.open)
     io.open = _file_tracker(io.open)
+    if hasattr(pathlib, "_normal_accessor"):
+        pathlib._normal_accessor.open = _file_tracker(pathlib._normal_accessor.open)
     sys.meta_path.insert(0, ImportTracker())
 
 
@@ -705,24 +709,39 @@ def setup_figure(width=1.0, height=1.0, columns=None, margin=False,
     """
     global _config, _interactive
 
-    # Need to install our file trackers (if desired)
-    # before we import Matplotlib.
-    if 'PGFUTILS_TRACK_FILES' in os.environ:
-        _install_standard_file_trackers()
-
     # Reset the configuration.
     _config_reset()
 
     # Load configuration from a local file if one exists.
     if os.path.exists('pgfutils.cfg'):
         _config.read('pgfutils.cfg')
+        _file_tracker.filenames.add(("r", "pgfutils.cfg"))
 
     # And anything given in the function call.
     if kwargs:
         _config.read_kwargs(**kwargs)
 
-    # Now we can add any extra trackers specified in the config.
+    # Set environment variables specified in the configuration.
+    for line in _config['pgfutils']['environment'].splitlines():
+        line = line.strip()
+        if not line:
+            continue
+
+        # Check the variables are formatted correctly.
+        if '=' not in line:
+            raise ValueError(
+                "Environment variables should be in the form NAME=VALUE. "
+                "The line '{}' does not match this.".format(line)
+            )
+
+        # And set them.
+        key, value = line.split("=", 1)
+        os.environ[key.strip()] = value.strip()
+
+    # Install file trackers if desired. This must be done before anything which
+    # imports Matplotlib.
     if 'PGFUTILS_TRACK_FILES' in os.environ:
+        _install_standard_file_trackers()
         extra = _config['pgfutils']['extra_tracking'].strip()
         if extra:
             _install_extra_file_trackers(extra.split(","))
@@ -756,19 +775,6 @@ def setup_figure(width=1.0, height=1.0, columns=None, margin=False,
     if not _interactive:                              # pragma: no cover
         matplotlib.use('pgf')
 
-    # Workaround time. The validator used on the preamble splits the value by
-    # commas, and the subsequent usage of it joins it again with newlines. This
-    # breaks a preamble containing commas (e.g., in arguments to a package).
-    # This will be fixed in later releases with a proper _validate_tex_preamble
-    # function. For older versions, replacing the validator with a call to
-    # str.splitlines() fixes the problem.
-    # https://github.com/matplotlib/matplotlib/pull/12674
-    # https://github.com/matplotlib/matplotlib/pull/12805
-    try:
-        from matplotlib.rcsetup import _validate_tex_preamble
-    except ImportError:  # pragma: no cover
-        matplotlib.rcParams.validate['pgf.preamble'] = str.splitlines
-
     # Specify which TeX engine we are using.
     matplotlib.rcParams['pgf.texsystem'] = _config['tex']['engine']
 
